'use strict';

var stream = require('stream');
var fs = require('fs');
var path = require('path');
var fse = require('fs-extra');
var utils = require('@strapi/utils');
var logging = require('@strapi/logger');

function _interopNamespaceDefault(e) {
  var n = Object.create(null);
  if (e) {
    Object.keys(e).forEach(function (k) {
      if (k !== 'default') {
        var d = Object.getOwnPropertyDescriptor(e, k);
        Object.defineProperty(n, k, d.get ? d : {
          enumerable: true,
          get: function () { return e[k]; }
        });
      }
    });
  }
  n.default = e;
  return Object.freeze(n);
}

var utils__namespace = /*#__PURE__*/_interopNamespaceDefault(utils);
var logging__namespace = /*#__PURE__*/_interopNamespaceDefault(logging);

const { PayloadTooLargeError, ApplicationError } = utils__namespace.errors;
const { kbytesToBytes, bytesToHumanReadable } = utils__namespace.file;
const logger = logging__namespace.createLogger();
const PUBLIC_FOLDER_NAME = 'public';
const UPLOADS_FOLDER_NAME = 'uploads';
var index = {
    init ({ sizeLimit: providerOptionsSizeLimit } = {}) {
        // TODO V5: remove providerOptions sizeLimit
        if (providerOptionsSizeLimit) {
            process.emitWarning('[deprecated] In future versions, "sizeLimit" argument will be ignored from upload.config.providerOptions. Move it to upload.config');
        }
        // Ensure uploads folder exists
        const uploadPath = path.resolve(strapi.dirs.static.public, UPLOADS_FOLDER_NAME);
        if (!fse.pathExistsSync(uploadPath)) {
            throw new Error(`The upload folder (${uploadPath}) doesn't exist or is not accessible. Please make sure it exists.`);
        }
        return {
            checkFileSize (file, options) {
                const { sizeLimit } = options ?? {};
                // TODO V5: remove providerOptions sizeLimit
                if (providerOptionsSizeLimit) {
                    if (kbytesToBytes(file.size) > providerOptionsSizeLimit) throw new PayloadTooLargeError(`${file.name} exceeds size limit of ${bytesToHumanReadable(providerOptionsSizeLimit)}.`);
                } else if (sizeLimit) {
                    if (kbytesToBytes(file.size) > sizeLimit) throw new PayloadTooLargeError(`${file.name} exceeds size limit of ${bytesToHumanReadable(sizeLimit)}.`);
                }
            },
            uploadStream (file) {
                if (!file.stream) {
                    return Promise.reject(new Error('Missing file stream'));
                }
                let uploadPath = getUploadPath(file.type, file.ownerId);
                let uploadUrl = getUploadUrl(file.type, file.ownerId);
                const { stream: stream$1 } = file;
                return new Promise((resolve, reject)=>{
                    stream.pipeline(stream$1, fs.createWriteStream(path.join(uploadPath, `${file.hash}${file.ext}`)), (err)=>{
                        if (err) {
                            return reject(err);
                        }
                        file.url = `/${uploadUrl}/${file.hash}${file.ext}`;
                        resolve();
                    });
                });
            },
            upload (file) {
                if (!file.buffer) {
                    return Promise.reject(new Error('Missing file buffer'));
                }
                let uploadPath = getUploadPath(file.type, file.ownerId);
                let uploadUrl = getUploadUrl(file.type, file.ownerId);
                const { buffer } = file;
                return new Promise((resolve, reject)=>{
                    // write file in public/assets folder
                    fs.writeFile(path.join(uploadPath, `${file.hash}${file.ext}`), buffer, (err)=>{
                        if (err) {
                            return reject(err);
                        }
                        file.url = `/${uploadUrl}/${file.hash}${file.ext}`;
                        resolve();
                    });
                });
            },
            delete (file) {
                let deletePath = getDeletePath(file);
                return new Promise((resolve, reject)=>{
                    const filePath = deletePath;
                    if (!fs.existsSync(filePath)) {
                        resolve("File doesn't exist");
                        return;
                    }
                    // remove file from public/assets folder
                    fs.unlink(filePath, (err)=>{
                        if (err) {
                            return reject(err);
                        }
                        resolve();
                    });
                });
            }
        };
    }
};
const getDeletePath = (file)=>{
    let deletePath;
    deletePath = path.resolve(strapi.dirs.static.public, '..');
    deletePath = deletePath + file.url;
    console.log('strapi-provider-upload-custom::getDeletePath ', deletePath);
    return deletePath;
};
const getUploadUrl = (type, ownerId)=>{
    let uploadUrl;
    const date = new Date();
    const currentYear = date.getFullYear().toString();
    if (type === 'private') {
        uploadUrl = type + '/' + UPLOADS_FOLDER_NAME + '/' + ownerId + '/' + currentYear;
    } else if (type === 'public' || type === 'API Uploads') {
        uploadUrl = PUBLIC_FOLDER_NAME + '/' + UPLOADS_FOLDER_NAME;
    }
    return uploadUrl;
};
const getUploadPath = (type, ownerId)=>{
    console.log('ownerId ', ownerId);
    console.log('type ', type);
    let uploadPath;
    const currentYear = new Date().getFullYear().toString();
    if (type === 'private') {
        uploadPath = path.resolve(strapi.dirs.static.public, '../private', UPLOADS_FOLDER_NAME, ownerId, currentYear);
    } else if (type === 'public' || type === 'API Uploads') {
        logger.warn('in public / API Uploads type');
        uploadPath = path.resolve(strapi.dirs.static.public, UPLOADS_FOLDER_NAME);
    } else {
        logger.warn('Provider: No matching type (private/public) found');
        //use strapi app error as bad request
        throw new ApplicationError('Bad Request: No matching type/folder (private, public, API Uploads) found', {
            status: 400
        });
    }
    console.log('strapi-provider-upload-custom::getUploadPath ', uploadPath);
    if (!checkIfPathExists(uploadPath)) {
        fse.mkdirpSync(uploadPath);
    }
    return uploadPath;
};
const checkIfPathExists = (uploadPath)=>{
    if (!fse.pathExistsSync(uploadPath)) {
        return false;
    } else {
        return true;
    }
};

module.exports = index;
//# sourceMappingURL=index.js.map
